// AttilaCoolMenu.h
/////////////////////////////////////////////////////////////////////////////
// This is part of the Attila library.
// Copyright (c) 1999, Attila contributors (see ReadMe.htm).
// All rights reserved. No warranties extended.
/////////////////////////////////////////////////////////////////////////////
// History:
//  -11/11/99:
//   Joe Mariadassou [JMariadassou@syssol.com.au] pointed out a Unicode
//   code having to do w/ sizeof.
//
//  -11/2/99:
//   Tommy Hui [thui@mediaone.net] pointed out a Unicode bug in DrawMenuText.

#pragma once
#ifndef INC_ATTILACOOLMENU
#define INC_ATTILACOOLMENU

#include "AttilaCommon.H"
#include "AttilaBars.H"

namespace Attila
{

/////////////////////////////////////////////////////////////////////////////
// Most of the stuff under here is ported from Paul Dilascia's most excellent 
// CoolMenu code of MSJ sample
//
// NOTE:In rel builds, when we disable all the default libraries in the project
// settings we still have to include libcmt.lib despite of using the intrinsic
// functions options(for memcpy et al) because of __chkstk compiler generated
// stack check routine for this file. #pragma for stack check has no effect as
// its not our code but some CRT prolog. Getting rid of that we can reduce the
// exe size from 28 kb to 2 kb lesser!

// All this is MFC's
#define RT_TOOLBAR					MAKEINTRESOURCE(241)
#define RGB_TO_RGBQUAD(r,g,b)		(RGB(b,g,r))
#define CLR_TO_RGBQUAD(clr)			(RGB(GetBValue(clr), GetGValue(clr), GetRValue(clr)))
#define _countof(array)				(sizeof(array)/sizeof(array[0]))


#ifndef OBM_CHECK
#define OBM_CHECK 32760 // from winuser.h
#endif

const CXGAP = 1;            // num pixels between button and text
const CXTEXTMARGIN = 2;     // num pixels after hilite to start text
const CXBUTTONMARGIN = 2;   // num pixels wider button is than bitmap
const CYBUTTONMARGIN = 2;   // ditto for height

// This is like MFCs res struct
struct TOOLBARDATA 
{
	WORD wVersion;		// version # should be 1
	WORD wWidth;		// width of one bitmap
	WORD wHeight;		// height of one bitmap
	WORD wItemCount;	// number of items
	WORD items[1];		// array of command IDs, actual size is wItemCount
//	WORD* items()
//	{ return (WORD*)(this+1); }
};

/////////////////////////////////////////////////////////////////////////////

class CCoolMenuMgr
{
public:
	CCoolMenuMgr():m_hFont(0)
	{
		m_szButton.cx = 0;
		m_szButton.cy = 0;
		m_szBitmap.cx = 0;
		m_szBitmap.cy = 0;
				
		NONCLIENTMETRICS info;
		info.cbSize = sizeof(info);
		::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(info), &info, 0);
		m_hFont = ::CreateFontIndirect(&info.lfMenuFont);
		
		for(int i = 0; i < 1024; i++)
		{
			m_arrResID[i] = -1;
		}
	}
	~CCoolMenuMgr()
	{
		if(m_hFont)
		{
			::DeleteObject(m_hFont);
			m_hFont = 0;
		}
		
		for(int i = 0; i < m_ImageList.GetImageCount(); i++)
		{
			m_ImageList.Remove(i);
		}
		m_ImageList.Destroy();
	}

public:
	//ala MFCs LoadToolbar
	BOOL LoadToolbar(UINT nIDToolbar)
	{
		// load bitmap
		HRSRC hRes = ::FindResource(_Module.GetResourceInstance(), MAKEINTRESOURCE(nIDToolbar), RT_BITMAP);
		if (hRes == NULL)
			return FALSE;

		HBITMAP hbmToolbar = LoadSysColorBitmap(_Module.GetResourceInstance(),hRes,FALSE);
		if (!hbmToolbar) {	return FALSE;	}
		
		// load toolbar
		HRSRC hRsrc = ::FindResource(_Module.GetResourceInstance(), MAKEINTRESOURCE(nIDToolbar), RT_TOOLBAR);
		if (hRsrc == NULL)
			return FALSE;

		HGLOBAL hg	= LoadResource(_Module.GetResourceInstance(), hRsrc);
		if (hg == NULL)
			return FALSE;

		TOOLBARDATA* ptbd = 0;
		ptbd = (TOOLBARDATA*)LockResource(hg);
		if (ptbd == NULL)
			return FALSE;
		
		// OK, I have the bitmap and toolbar. 
		SIZE sz = { ptbd->wWidth, ptbd->wHeight};
		if (m_szBitmap.cx==0)
		{
			// First toolbar: initialized bitmap/button sizes and create image list.
			m_szBitmap.cx = sz.cx;
			m_szBitmap.cy = sz.cy;
			
			m_szButton.cx = sz.cx +  (CXBUTTONMARGIN<<1);
			m_szButton.cy = sz.cy +  (CYBUTTONMARGIN<<1);
				
			m_ImageList.Create(sz.cx, sz.cy, ILC_MASK, 0, 10);
		} 
		else if (!(m_szBitmap.cx == sz.cx &&  m_szBitmap.cy == sz.cy) ) 
		{
			// button sizes different -- oops
			UnlockResource(hg); 
			FreeResource(hg); 
			::DeleteObject(hbmToolbar);
			hbmToolbar = 0;
			return FALSE;
		}

		// I have a good toolbar: now add bitmap to the image list, and each
		// command ID to m_arrResID array. Note that LoadSysColorBitmap will
		// change gray -> COLOR_3DFACE, so use that for image list background.
		WORD iNextImage = m_ImageList.GetImageCount();
		m_ImageList.Add(hbmToolbar, GetSysColor(COLOR_3DFACE));
		for (int i = 0; i < ptbd->wItemCount; i++) 
		{
			UINT nID = ptbd->items[i];
			if (nID > 0) 
			{
				if (GetButtonIndex(nID) >= 0) 
				{
					UnlockResource(hg); 
					FreeResource(hg); 
					::DeleteObject(hbmToolbar);hbmToolbar = 0;
				} 
				else
				{
					//m_mapIDtoImage.Add(nID,iNextImage++);
					m_arrResID[iNextImage++] = nID;
				}
			}
		}
		UnlockResource(hg); 
		FreeResource(hg); 
		::DeleteObject(hbmToolbar);
		hbmToolbar = 0;
		return TRUE; 
	}

	BOOL OnMenuSelect(UINT nItemID, UINT nFlags, HMENU hSysMenu)
	{
		if (hSysMenu==NULL && nFlags==0xFFFF) 
		{
			// Windows has closed the menu: restore all menus to original state
			for(int i=0; i < m_menuList.GetSize(); i++ )
			{
				ConvertMenu(m_menuList[i],0, FALSE, FALSE);
			}
			
			if( m_menuList.GetSize() )
				m_menuList.RemoveAll();
		}
		return FALSE;
	}
	
	BOOL OnInitMenuPopup(HMENU hmenuPopup, UINT uPos,BOOL bSystemMenu)
	{
		ConvertMenu(hmenuPopup, uPos, bSystemMenu, TRUE);
		return TRUE;
	}

	BOOL OnMeasureItem(MEASUREITEMSTRUCT* pMI)
	{
		if(!pMI){ return FALSE; }

		CPrivateData* pmd = (CPrivateData*)pMI->itemData;
		if (pMI->CtlType != ODT_MENU || !pmd->IsItMine())
			return FALSE; // not handled by me

		if (pmd->m_fType & MFT_SEPARATOR) 
		{
			// separator: use half system height and zero width
			pMI->itemHeight = GetSystemMetrics(SM_CYMENU)>>1;
			pMI->itemWidth  = 0;

		} 
		else
		{
			// compute size of text: use DrawText with DT_CALCRECT
			HDC hdc = GetWindowDC(NULL);	// screen DC--I won't actually draw on it
			RECT rcText = {0,0,0,0};
			HFONT hOldFont = (HFONT)::SelectObject(hdc, m_hFont);
			
			::DrawText(hdc,pmd->m_psz,lstrlen(pmd->m_psz)+1,&rcText, DT_SINGLELINE|DT_LEFT|DT_VCENTER|DT_CALCRECT);

			::SelectObject(hdc,hOldFont);
			::ReleaseDC(0,hdc);hdc = 0;
			
			// height of item is just height of a standard menu item
			pMI->itemHeight= max(GetSystemMetrics(SM_CYMENU), rcText.bottom - rcText.top);

			// width is width of text plus a bunch of stuff
			int cx = rcText.right - rcText.left;	// text width 
			cx += CXTEXTMARGIN<<1;					// L/R margin for readability
			cx += CXGAP;							// space between button and menu text
			cx += m_szButton.cx<<1;					// button width (L=button; R=empty margin)
			
			// whatever value I return in lpms->itemWidth, Windows will add the
			// width of a menu checkmark, so I must subtract to defeat Windows. Argh.
			cx -= GetSystemMetrics(SM_CXMENUCHECK)-1;
			
			pMI->itemWidth = cx;		// done deal
		}
		return TRUE; // handled
	}
	BOOL OnDrawItem(DRAWITEMSTRUCT* pDI)
	{
		if(!pDI){ return FALSE; }
		
		CPrivateData* pmd = (CPrivateData*)pDI->itemData;
		if (pDI->CtlType != ODT_MENU || !pmd->IsItMine())
			return FALSE; // not handled by me
		
		HDC hdc = pDI->hDC;
		RECT rcItem;::SetRect(&rcItem,pDI->rcItem.left,pDI->rcItem.top,pDI->rcItem.right,pDI->rcItem.bottom);

		if (pmd->m_fType & MFT_SEPARATOR) // draw separator
		{
			RECT rc;::SetRect(&rc,rcItem.left,rcItem.top,rcItem.right,rcItem.bottom);		// copy rect
			rc.top += (rc.bottom - rc.top)>>1;											// vertical center
			::DrawEdge(hdc, &rc, EDGE_ETCHED, BF_TOP);								// draw separator line
		} 
		else // not a separator
		{													
			BOOL bDisabled	=	pDI->itemState & ODS_GRAYED;
			BOOL bSelected	=	pDI->itemState & ODS_SELECTED;
			BOOL bChecked	=	pDI->itemState & ODS_CHECKED;
			BOOL bHaveButn	=	FALSE;

			// Paint button, or blank if none
			RECT rcButn;
			rcButn.left		= rcItem.left;
			rcButn.top		= rcItem.top;
			rcButn.right	= rcButn.left + m_szButton.cx;
			rcButn.bottom	= rcButn.top + m_szButton.cy;

			// center vertically
			POINT pt; pt.x = 0;
			int nY = (rcItem.bottom - rcItem.top) - (rcButn.bottom - rcButn.top);
			pt.y = nY>>1;

			rcButn.left += pt.x;
			rcButn.top  += pt.y;

			int iButton = pmd->m_nButton;
			if (iButton >= 0) 
			{
				// this item has a button!
				bHaveButn = TRUE;

				// compute point to start drawing
				SIZE sz = { (rcButn.right - rcButn.left) - m_szBitmap.cx, (rcButn.bottom - rcButn.top) - m_szBitmap.cy };
				sz.cx >>= 1;
				sz.cy >>= 1;
				POINT p = {rcButn.left + sz.cx, rcButn.top + sz.cy };
				
				// draw disabled or normal
				if (!bDisabled) 
				{
					// normal: fill BG depending on state
					FillRect(hdc, rcButn, GetSysColor((bChecked && !bSelected) ? COLOR_3DLIGHT : COLOR_MENU));

					// draw pushed-in or popped-out edge
					if (bSelected || bChecked) 
					{
						RECT rc2;::SetRect(&rc2,rcButn.left,rcButn.top,rcButn.right,rcButn.bottom);
						
						::DrawEdge(hdc,&rc2, bChecked ? BDR_SUNKENOUTER : BDR_RAISEDINNER,BF_RECT);
					}
					// draw the button!
					m_ImageList.Draw(hdc,iButton,p, ILD_TRANSPARENT);
				} 
				else
				{
					// use DrawEmbossed to draw disabeld button, w/color flag
					DrawEmbossed(hdc, m_ImageList, iButton, p,FALSE);
				}
			}
			else 
			{
				// no button: look for custom checked/unchecked bitmaps
				CMenuItemInfo info;
				info.fMask = MIIM_CHECKMARKS;
				::GetMenuItemInfo((HMENU)pDI->hwndItem,pDI->itemID, MF_BYCOMMAND, &info);
				if (bChecked || info.hbmpUnchecked) 
				{
					bHaveButn = Draw3DCheckmark(hdc, rcButn, bSelected,bChecked ? info.hbmpChecked : info.hbmpUnchecked);
				}
			}

			// Done with button, now paint text. First do background if needed.
			int cxButn = m_szButton.cx;				// width of button
			COLORREF colorBG = ::GetSysColor(bSelected ? COLOR_HIGHLIGHT : COLOR_MENU);
			if (bSelected || pDI->itemAction==ODA_SELECT) 
			{
				// selected or selection state changed: paint text background
				RECT rcBG = {rcItem.left,rcItem.top,rcItem.right,rcItem.bottom};	// whole rectangle
				if (bHaveButn)									// if there's a button:
				{
					rcBG.left += cxButn + CXGAP;				//  don't paint over it!
				}
				FillRect(hdc, rcBG, colorBG);					// paint it!
			}

			// compute text rectangle and colors
			RECT rcText = {rcItem.left,rcItem.top,rcItem.right,rcItem.bottom};	// start w/whole item
			rcText.left += cxButn + CXGAP + CXTEXTMARGIN;		// left margin
			rcText.right -= cxButn;								// right margin
			::SetBkMode(hdc,TRANSPARENT);							// paint transparent text

			// MW - Choose your colors but choose carefully
			COLORREF colorText;
			if( bDisabled && bSelected )
				colorText = ::GetSysColor(COLOR_3DLIGHT);
			else if( bDisabled )
				colorText = ::GetSysColor(COLOR_GRAYTEXT);
			else if( bSelected )
				colorText = ::GetSysColor(COLOR_HIGHLIGHTTEXT);
			else
				colorText = ::GetSysColor(COLOR_MENUTEXT);

			// Now paint menu item text.	No need to select font,
			// because windows sets it up before sending WM_DRAWITEM
			if (bDisabled && (!bSelected || colorText == colorBG)) 
			{
				// disabled: draw hilite text shifted southeast 1 pixel for embossed
				// look. Don't do it if item is selected, tho--unless text color same
				// as menu highlight color. Got it?
				RECT rcText2;::SetRect(&rcText2,rcText.left + 1,rcText.top + 1,rcText.right,rcText.bottom);
				DrawMenuText(hdc, rcText2, pmd->m_psz,GetSysColor(COLOR_3DHILIGHT));
			}
			DrawMenuText(hdc, rcText, pmd->m_psz, colorText); // finally!*/
		}
		return TRUE; // handled
	}

private://Ditto MFC AfxLoadSysColorBitmap()
	HBITMAP  LoadSysColorBitmap(HINSTANCE hInst, HRSRC hRsrc, BOOL bMono)
	{
		struct AFX_COLORMAP
		{
			// use DWORD instead of RGBQUAD so we can compare two RGBQUADs easily
			DWORD rgbqFrom;
			int iSysColorTo;
		};

		const AFX_COLORMAP _afxSysColorMap[] =
		{
			// mapping from color in DIB to system color
			{ RGB_TO_RGBQUAD(0x00, 0x00, 0x00),  COLOR_BTNTEXT },       // black
			{ RGB_TO_RGBQUAD(0x80, 0x80, 0x80),  COLOR_BTNSHADOW },     // dark gray
			{ RGB_TO_RGBQUAD(0xC0, 0xC0, 0xC0),  COLOR_BTNFACE },       // bright gray
			{ RGB_TO_RGBQUAD(0xFF, 0xFF, 0xFF),  COLOR_BTNHIGHLIGHT }   // white
		};

		HGLOBAL hglb = 0;
		if ((hglb = LoadResource(hInst, hRsrc)) == NULL)
			return NULL;

		LPBITMAPINFOHEADER lpBitmap = (LPBITMAPINFOHEADER)LockResource(hglb);
		if (lpBitmap == NULL)
			return NULL;

		// make copy of BITMAPINFOHEADER so we can modify the color table
		const int nColorTableSize = 16;
		UINT nSize = lpBitmap->biSize + nColorTableSize * sizeof(RGBQUAD);
						
		LPBITMAPINFOHEADER lpBitmapInfo = (LPBITMAPINFOHEADER)malloc(nSize);
		if (lpBitmapInfo == NULL)
			return NULL;
		memcpy(lpBitmapInfo, lpBitmap, nSize);

		// color table is in RGBQUAD DIB format
		DWORD* pColorTable = (DWORD*)(((LPBYTE)lpBitmapInfo) + (UINT)lpBitmapInfo->biSize);

		for (int iColor = 0; iColor < nColorTableSize; iColor++)
		{
			// look for matching RGBQUAD color in original
			for (int i = 0; i < _countof(_afxSysColorMap); i++)
			{
				if (pColorTable[iColor] == _afxSysColorMap[i].rgbqFrom)
				{
					if (bMono)
					{
						// all colors except text become white
						if (_afxSysColorMap[i].iSysColorTo != COLOR_BTNTEXT)
							pColorTable[iColor] = RGB_TO_RGBQUAD(255, 255, 255);
					}
					else
						pColorTable[iColor] =
							CLR_TO_RGBQUAD(::GetSysColor(_afxSysColorMap[i].iSysColorTo));
					break;
				}
			}
		}

		int nWidth = (int)lpBitmapInfo->biWidth;
		int nHeight = (int)lpBitmapInfo->biHeight;
		HDC hDCScreen = ::GetDC(NULL);
		HBITMAP hbm = ::CreateCompatibleBitmap(hDCScreen, nWidth, nHeight);

		if (hbm != NULL)
		{
			HDC hDCGlyphs = ::CreateCompatibleDC(hDCScreen);
			HBITMAP hbmOld = (HBITMAP)::SelectObject(hDCGlyphs, hbm);

			LPBYTE lpBits;
			lpBits = (LPBYTE)(lpBitmap + 1);
			lpBits += (1 << (lpBitmapInfo->biBitCount)) * sizeof(RGBQUAD);

			StretchDIBits(hDCGlyphs, 0, 0, nWidth, nHeight, 0, 0, nWidth, nHeight,lpBits, (LPBITMAPINFO)lpBitmapInfo, DIB_RGB_COLORS, SRCCOPY);
			SelectObject(hDCGlyphs, hbmOld);
		
			::DeleteDC(hDCGlyphs);
		}
		::ReleaseDC(NULL, hDCScreen);

		// free copy of bitmap info struct and resource itself
		free(lpBitmapInfo);
		::FreeResource(hglb);

		return hbm;
		
	}

	int  GetButtonIndex(UINT nID) 
	{
		for(int i = 0; i < 1024; i++)
		{
			if(m_arrResID[i] == nID)
			{
				return i;
			}
		}
		return -1;
	}
	void FillRect(HDC& hdc, const RECT& rc, COLORREF color)
	{
		HBRUSH hBr = ::CreateSolidBrush(color);
		HBRUSH hOldBrush = (HBRUSH)::SelectObject(hdc,hBr);
		::PatBlt(hdc,rc.left, rc.top, (rc.right - rc.left), (rc.bottom - rc.top), PATCOPY);
		::SelectObject(hdc,hOldBrush);
		::DeleteObject(hBr);hBr = 0;
	}
	void DrawEmbossed(HDC& hdc, CImageList& il,int i,POINT& p, BOOL bColor)
	{
		IMAGEINFO info;
		il.GetImageInfo(0, &info);
		RECT rc; ::SetRect(&rc,info.rcImage.left,info.rcImage.top,info.rcImage.right,info.rcImage.bottom);
		
		int cx = rc.right - rc.left;
		int cy = rc.bottom - rc.top;

		// create memory dc
		HDC hdcMem = ::CreateCompatibleDC(hdc);

		// create mono or color bitmap
		HBITMAP hBmp = 0;
		if (bColor)
			hBmp = ::CreateCompatibleBitmap(hdc, cx, cy);
		else
			hBmp = ::CreateBitmap(cx, cy, 1, 1, NULL);

		// draw image into memory DC--fill BG white first
		HBITMAP hOldBitmap = (HBITMAP)::SelectObject(hdcMem,hBmp);
		
		::PatBlt(hdcMem,0, 0, cx, cy, WHITENESS);
		POINT pt = {0,0};
		il.Draw(hdcMem, i, pt, ILD_TRANSPARENT);

		// This seems to be required. Why, I don't know. ???
		COLORREF colorOldBG = ::SetBkColor(hdc,RGB(255,255,255)); // white

		// Draw using hilite offset by (1,1), then shadow
		HBRUSH hbrShadow = CreateSolidBrush(GetSysColor(COLOR_3DSHADOW));
		HBRUSH hbrHilite = CreateSolidBrush(GetSysColor(COLOR_3DHIGHLIGHT));
						
		HBRUSH hOldBrush = (HBRUSH)::SelectObject(hdc,hbrHilite);
		
		const DWORD TRANSPARENTROP = 0xb8074a;

		::BitBlt(hdc,p.x+1, p.y+1, cx, cy, hdcMem, 0, 0, TRANSPARENTROP);
		::SelectObject(hdc,hbrShadow);
		::BitBlt(hdc,p.x, p.y, cx, cy, hdcMem, 0, 0, TRANSPARENTROP);
		::SelectObject(hdc,hOldBrush);
		
		::DeleteObject(hbrShadow);
		::DeleteObject(hbrHilite);

		::SetBkColor(hdc,colorOldBG);				 // restore
		SelectObject(hdcMem,hOldBitmap);		 // ...
		DeleteObject(hBmp);
		
		DeleteDC(hdcMem);
	}

	void DrawMenuText(HDC& hdc, RECT& rc, LPTSTR psz,COLORREF color)
	{
		ATLASSERT(psz);

		::SetTextColor(hdc,color);

		TCHAR szBuffer[1024], *pTab = NULL;
		
		_tcsncpy(szBuffer, psz, sizeof(szBuffer)/sizeof(*szBuffer));
		pTab = _tcstok(szBuffer, _T("\t"));

		DrawText(hdc, pTab, lstrlen(pTab), &rc, DT_SINGLELINE | DT_VCENTER | DT_LEFT);
		pTab = _tcstok(NULL, _T("\t"));
		if( pTab )
			DrawText(hdc, pTab, lstrlen(pTab), &rc, DT_SINGLELINE | DT_VCENTER | DT_RIGHT);
	}

	// Draw 3D checkmark:
	//		rc				rectangle to center bitmap in
	//		bSelected	TRUE if button is also selected
	//		hbmCheck		Checkmark bitmap to use, or NULL for default
	BOOL Draw3DCheckmark(HDC& hdc, const RECT& rc,BOOL bSelected, HBITMAP hbmCheck)
	{
		// get checkmark bitmap if none, use Windows standard
		bool bCreated = false;
		if (!hbmCheck) 
		{
			hbmCheck = ::LoadBitmap(NULL, MAKEINTRESOURCE(OBM_CHECK));
			bCreated = true;
		}
		
		// center bitmap in caller's rectangle
		BITMAP bm;
		::GetObject(hbmCheck, sizeof(bm), &bm);
		int cx = bm.bmWidth;
		int cy = bm.bmHeight;
		RECT rcDest;::SetRect(&rcDest,rc.left,rc.top,rc.right,rc.bottom);
		
		POINT p = {0,0};
		SIZE delta = { (rc.right - rc.left - cx)/2, (rc.bottom - rc.top - cy)/2 };
		
		if (rc.right - rc.left > cx)
		{
			rcDest.left = rc.left + delta.cx; 
			rcDest.top = rc.top + delta.cx;
			rcDest.right = rcDest.left + cx;
			rcDest.bottom = rcDest.top + cy;
		}
		else
		{
			p.x -= delta.cx;
			p.y -= delta.cy;
		}

		// select checkmark into memory DC
		HDC hDCMem = ::CreateCompatibleDC(hdc);
		HBITMAP hOldBM = (HBITMAP)::SelectObject(hDCMem , hbmCheck);

		// set BG color based on selected state
		COLORREF colorOld = ::SetBkColor(hdc, GetSysColor(bSelected ? COLOR_MENU : COLOR_3DLIGHT));
		::BitBlt(hdc,rcDest.left, rcDest.top, (rcDest.right - rcDest.left), (rcDest.bottom - rcDest.top),hDCMem, p.x, p.y, SRCCOPY);
		::SetBkColor(hdc,colorOld);

		::SelectObject(hDCMem, hOldBM); // restore

		// draw pushed-in hilight.
		if ((rc.right - rc.left) > cx)				// if room:
			::InflateRect(&rcDest,1,1);	// inflate checkmark by one pixel all around

		::DrawEdge(hdc,&rcDest, BDR_SUNKENOUTER, BF_RECT);
		
		DeleteDC(hDCMem); 
		if(bCreated)
		{
			::DeleteObject(hbmCheck);
		}
		return TRUE;
	}

	// This rather gnarly function is used both to convert the menu from strings to
	// owner-draw and vice versa. In either case, it also appends automagic
	// accelerator key names to the menu items, if m_bAutoAccel is TRUE.
	void ConvertMenu(HMENU& hMenu,UINT nIndex, BOOL bSysMenu, BOOL bShowButtons)
	{
		TCHAR pszItemName[1024];
		UINT nItem = ::GetMenuItemCount(hMenu);
		for (UINT i = 0; i < nItem; i++) // loop over each item in menu
		{	
			// get menu item info
			TCHAR itemname[256];
			CMenuItemInfo info;
			info.fMask = MIIM_SUBMENU | MIIM_DATA | MIIM_ID | MIIM_TYPE;
			info.dwTypeData = itemname;
			info.cch = sizeof(itemname)/sizeof(*itemname);
			::GetMenuItemInfo(hMenu, i, TRUE, &info);
			CPrivateData* pmd = (CPrivateData*)info.dwItemData;
			if (pmd && !pmd->IsItMine()) 
			{
				continue; // owner-draw menu item isn't mine--leave it alone
			}

			if (bSysMenu && info.wID >= 0xF000) 
			{
				continue; // don't do for system menu commands
			}
		
			// now that I have the info, I will modify it
			info.fMask = 0;	// assume nothing to change

			if (bShowButtons) 
			{
				// I'm showing buttons: convert to owner-draw
				if (!(info.fType & MFT_OWNERDRAW)) 
				{
					// If not already owner-draw, make it so. NOTE: If app calls
					// pCmdUI->SetText to change the text of a menu item, MFC will
					// turn the item to MFT_STRING. So I must set it back to
					// MFT_OWNERDRAW again. In this case, the menu item data (pmd)
					// will still be there.
					info.fType |= MFT_OWNERDRAW;
					info.fMask |= MIIM_TYPE;
					if (!pmd) // if no item data:
					{									
						pmd = new CPrivateData();							//   create one
						pmd->m_fType = info.fType;						//   handy when drawing
						pmd->m_nButton = GetButtonIndex(info.wID);
						info.dwItemData = (DWORD)pmd;					//   set in menu item data
						info.fMask |= MIIM_DATA;						//   set item data
					}

					// MW - We do not neglect the accelerators 
					if (info.dwTypeData)
						lstrcpy(pmd->m_psz, info.dwTypeData);	// copy menu item string
				}
				
				if (m_menuList.Find(hMenu) == -1)
					m_menuList.Add(hMenu);

			} 
			else // no buttons -- I'm converting to strings 
			{
				if (info.fType & MFT_OWNERDRAW)							// if ownerdraw:
				{	
					info.fType &= ~MFT_OWNERDRAW;						//   turn it off
					info.fMask |= MIIM_TYPE;							//   change item type
					lstrcpy(pszItemName,pmd->m_psz);					//   save name before deleting pmd
				} 
				else													// otherwise:
				{
					lstrcpy(pszItemName,info.dwTypeData);						//   use name from MENUITEMINFO
				}

				if (pmd) 
				{
					info.dwItemData = NULL;				// item data is NULL
					info.fMask |= MIIM_DATA;			// change it
					delete pmd;							// and item data too
				}
	
				if (info.fMask & MIIM_TYPE) 
				{
					lstrcpyn(itemname, pszItemName, sizeof(itemname)/sizeof(*itemname));
					info.dwTypeData = itemname;
					info.cch = lstrlen(pszItemName);
				}
			}

			// if after all the above, there is anything to change, change it
			if (info.fMask) 
			{
				SetMenuItemInfo(hMenu, i, TRUE, &info);
			}
		}
	}

	TCHAR* strtok (TCHAR* string,const TCHAR* control)
	{
			TCHAR *str;
			const TCHAR *ctrl = control;

			TCHAR map[32];
			int count;

			static TCHAR *nextoken;

			/* Clear control map */
			for (count = 0; count < 32; count++)
					map[count] = 0;

			/* Set bits in delimiter table */
			do 
			{
				map[*ctrl >> 3] |= (1 << (*ctrl & 7));

			} while (*ctrl++);

			// Initialize str. If string is NULL, set str to the saved
			// pointer (i.e., continue breaking tokens out of the string
			// from the last strtok call)
			if (string)
				str = string;
			else
				str = nextoken;

			// Find beginning of token (skip over leading delimiters). Note that
			// there is no token iff this loop sets str to point to the terminal
			// null (*str == '\0')
			while ( (map[*str >> 3] & (1 << (*str & 7))) && *str )
					str++;

			string = str;

			// Find the end of the token. If it is not the end of the string,
			// put a null there.
			for ( ; *str ; str++ )
			{
				if ( map[*str >> 3] & (1 << (*str & 7)) ) 
				{
						*str++ = '\0';
						break;
				}
			}
			// Update nextoken (or the corresponding field in the per-thread data
			// structure
			nextoken = str;
			// Determine if a token has been found.
			if ( string == str )
				return NULL;
			else
				return string;
	}

private:
	CImageList									m_ImageList;
	HFONT										m_hFont;
	SIZE										m_szButton;
	SIZE										m_szBitmap;
	UINT										m_arrResID[1024];
	CSimpleArray<HMENU>							m_menuList;
	
	struct CPrivateData
	{
		long		m_nCookie;		
		TCHAR		m_psz[1024];
		UINT		m_fType;			// original item type flags
		int			m_nButton;			// index of button image in image list
		CPrivateData():m_nCookie(0x9989),m_nButton(0),m_fType(0){}
		bool IsItMine(){ return m_nCookie == 0x9989; }
	};

	struct CMenuItemInfo : public MENUITEMINFO 
	{
		CMenuItemInfo()
		{ memset(this, 0, sizeof(MENUITEMINFO));
		  cbSize = sizeof(MENUITEMINFO);
		}
	};
};

/////////////////////////////////////////////////////////////////////////////

template <typename TBase = CToolBarCtrl, typename TWinTraits = CMenuBarTraits, UINT nResID = 0>
class CCoolMenuBarImplT : public CMenuBarImplT<TBase,TWinTraits> 
{
public:
	typedef CCoolMenuBarImplT<TBase,TWinTraits >			thisClass;
	typedef CMenuBarImplT<TBase,TWinTraits>					baseClass;

	CCoolMenuBarImplT()
	{
//		m_CoolMgr.LoadToolbar(nResID);
	}
	~CCoolMenuBarImplT()
	{
		
	}

public:	
	BEGIN_MSG_MAP(thisClass)
		MESSAGE_HANDLER(WM_INITMENUPOPUP, OnInitMenuPopUp)				
		MESSAGE_HANDLER(WM_MEASUREITEM, OnMeasureItem)
		MESSAGE_HANDLER(WM_DRAWITEM, OnDrawItem)
		MESSAGE_HANDLER(WM_MENUSELECT, OnMenuSelect)
		CHAIN_MSG_MAP(baseClass)
	END_MSG_MAP()

public:

	HWND Create(HWND hWndParent, RECT &rcPos, UINT nID = 0)
	{
		m_CoolMgr.LoadToolbar(nID);
		return baseClass::Create(hWndParent,rcPos,nID);
	}	
	//msg handlers
	// JR - added bHandled = FALSE because it was eating the menu messages
	//  We need access to them for Command Update UI
	

	LRESULT OnInitMenuPopUp(UINT,WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;
		m_CoolMgr.OnInitMenuPopup((HMENU) wParam,(UINT) LOWORD(lParam),(BOOL) HIWORD(lParam) );
		return 0L;
	}

	LRESULT OnMeasureItem(UINT,WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;
		m_CoolMgr.OnMeasureItem((MEASUREITEMSTRUCT*)lParam);
		return 0L;
	}

	LRESULT OnDrawItem(UINT,WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;
		m_CoolMgr.OnDrawItem((DRAWITEMSTRUCT*)lParam);
		return 0L;
	}
	LRESULT OnMenuSelect(UINT,WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;
		m_CoolMgr.OnMenuSelect((UINT)LOWORD(wParam), (UINT)HIWORD(wParam), (HMENU)lParam);
		return 0L;
	}
protected:
	CCoolMenuMgr m_CoolMgr;
};

typedef CCoolMenuBarImplT<CToolBarCtrl, CMenuBarTraits> CCoolMenuBar;
typedef CToolPanelT<CCoolMenuBar,CToolBar>				CCoolToolPanel;	

}   // namespace Attila

#ifndef _ATTILA_NOUSE_NAMESPACE
using namespace Attila;
#endif

#endif  // INC_ATTILACOOLMENU
