// AttilaModules.h
/////////////////////////////////////////////////////////////////////////////
// This is part of the Attila library.
// Copyright (c) 1999, Attila contributors (see ReadMe.htm).
// All rights reserved. No warranties extended.
/////////////////////////////////////////////////////////////////////////////
// History:
//  -11/2/99:
//   Tommy Hui [thui@mediaone.net] fixed a Unicode bug.

#pragma once
#ifndef INC_ATTILAMISC
#define INC_ATTILAMODULES

#include <ddeml.h>
#include "smalldde.h"

namespace Attila
{

/////////////////////////////////////////////////////////////////////////////
// CComSingleInstanceModule

class CComSingleInstanceModuleBase
{
public:
    CComSingleInstanceModuleBase() : m_bIs2ndInstance(false)
    {
        ATLASSERT(!s_pModule);
        s_pModule = this;
    }

    HRESULT Init(LPCTSTR pszCmdLine, PFNCALLBACK pfcCallback)
    {
        DDER(m_dde.Initialize(pfcCallback, 0));
        DDER(m_hszApp.Create(m_dde, __T("Single_Instance_App")));    // TODO: Base this on appname

        CDdeString  hszTopic;
        DDER(hszTopic.Create(m_dde, __T("System")));

        CDdeConversation    conv;
        long err = conv.Connect(m_dde, m_hszApp, hszTopic);

        // 1st instance already running
        if( conv )
        {
            // Signal that we're supposed to shut down
            m_bIs2ndInstance = true;

            // Send command line to 2nd instance
            USES_CONVERSION;
            DDER(CDdeData().ClientTransaction(m_dde, conv, T2CA(pszCmdLine)));

            return S_OK;
        }

        // We're the 1st instance, so register as DDE application
        // for the 2nd instance to pass us the command line
        DDER(m_data.NameService(m_dde, m_hszApp, DNS_REGISTER));

        return S_OK;
    }

	void Term()
    {
        if( m_data ) m_data.NameService(m_dde, m_hszApp, DNS_UNREGISTER);
    }

    DWORD GetDdeInstanceId()
    {
        return m_dde;
    }

    virtual void On2ndInstance(LPCTSTR pszCmdLine) =0;

    static
    HDDEDATA CALLBACK DdeCallback(UINT nType, UINT, HCONV, HSZ hszTopic, HSZ, HDDEDATA hDdeData, DWORD, DWORD)
    {
        switch( nType )
        {
        case XTYP_CONNECT:
        {
            ATLASSERT(s_pModule);

            DWORD       id = s_pModule->GetDdeInstanceId();
            TCHAR       szSystem[] = __T("System");
            const UINT  cb = sizeof(szSystem)/sizeof(*szSystem);
            TCHAR       szTopic[cb];

            if( !DdeQueryString(id, hszTopic, szTopic, cb, CP_WINNEUTRAL) ||
                lstrcmp(szTopic, szSystem) != 0 )
            {
                return (HDDEDATA)FALSE;
            }

            return (HDDEDATA)TRUE;
        }
        break;

        case XTYP_EXECUTE:
        {
            UINT    cb = DdeGetData(hDdeData, 0, 0, 0) + sizeof(TCHAR);
            LPTSTR  psz = new TCHAR[cb];

            if( !DdeGetData(hDdeData, (BYTE*)psz, cb, 0) ) return FALSE;

            ATLASSERT(s_pModule);
            s_pModule->On2ndInstance(psz);
            delete[] psz;
        }
        break;
        }
        
        return 0;
    }

    bool Is2ndInstance()
    {
        return m_bIs2ndInstance;
    }
    
private:
    bool        m_bIs2ndInstance;
    CDde        m_dde;
    CDdeString  m_hszApp;
    CDdeData    m_data;

    static CComSingleInstanceModuleBase*    s_pModule;
};

__declspec(selectany) CComSingleInstanceModuleBase* CComSingleInstanceModuleBase::s_pModule = 0;

template <typename Deriving>
class CComSingleInstanceModule : public CComSingleInstanceModuleBase
{
public:
    HRESULT Init(LPCTSTR pszCmdLine)
    {
        return CComSingleInstanceModuleBase::Init(pszCmdLine, static_cast<Deriving*>(this)->GetDdeCallback());
    }

    void Term()
    {
        CComSingleInstanceModuleBase::Term();
    }

    PFNCALLBACK GetDdeCallback()
    {
        return DdeCallback;
    }
};

/////////////////////////////////////////////////////////////////////////////
// Multi-SDI App Thread Manager

class CThreadMgr
{
public:
	CThreadMgr() : m_pThreadProc(0)
	{
		m_hDone = CreateEvent(NULL, TRUE, FALSE, NULL ) ;
		m_cs.Init() ;
	}

	~CThreadMgr()
	{
		CloseHandle(m_hDone) ;
		m_cs.Term() ;
	}

	HRESULT Init(LPTHREAD_START_ROUTINE threadproc)
	{
		m_cs.Lock() ;
		m_pThreadProc = threadproc ;
		m_cs.Unlock() ;
        return S_OK;
	}

	DWORD CreateNewThread()
	{
		m_cs.Lock() ;
		DWORD dwRet = 0 ;
		if ( 0 == m_pThreadProc )
			ATLTRACE(_T("Must call CThreadMgr::Init() Before CThreadMgr::CreateNewThread()\n")) ;
		else
		{
			DWORD dwThreadId = 0;
			HANDLE h = ::CreateThread(NULL, 0, m_pThreadProc, this, 0, &dwThreadId);
			CloseHandle(h) ; // we don't need the handle anymore
			m_dwThreadIds.Add(dwThreadId) ;
			dwRet = dwThreadId;
		}
		m_cs.Unlock();
		return dwRet ;
	}

	BOOL WaitTilDone()
	{
		return AtlWaitWithMessageLoop(m_hDone) ;
	}

	void ThreadTerm()
	{
		Remove(::GetCurrentThreadId()) ;
	}

protected:	

	void Remove(DWORD dwId)
	{
		m_cs.Lock() ;
		int dwThreadCount = m_dwThreadIds.GetSize() ;
		for ( int dw = 0 ; dw < dwThreadCount ; dw++ )
		{
			if ( m_dwThreadIds[dw] == dwId )
			{
				if(dw < dwThreadCount - 1)
					m_dwThreadIds[dw] = m_dwThreadIds[dwThreadCount -1];

				m_dwThreadIds.RemoveAt(dwThreadCount-1) ;

				if ( 1 == dwThreadCount ) // just removed the last thread ?
					SetEvent(m_hDone) ;
				break ;
			}
		}
		m_cs.Unlock() ;
	}

protected:
	CSimpleArray<DWORD>			m_dwThreadIds; 
	LPTHREAD_START_ROUTINE  	m_pThreadProc;
	CComCriticalSection			m_cs ;
	HANDLE						m_hDone ;
};

/////////////////////////////////////////////////////////////////////////////
// Multi-SDI Module

class CMultiSDIModule :  public CThreadMgr
{
public:
    void Term()
    {
    }
};

}   // namespace Attila

#ifndef _ATTILA_NOUSE_NAMESPACE
using namespace Attila;
#endif

#endif //INC_ATTILAMODULES
